
#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>

#define	LCD_CD	PC7   
#define	LCD_WR  PC6
#define	LCD_RD  PC5
#define	LCD_CS	PC4
#define	LCD_RES	PC2

/* macros for control lines */                                  

#define CLR_CD PORTC &= ~(1<<LCD_CD);
#define SET_CD PORTC |=  (1<<LCD_CD);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RES);
#define SET_RESET PORTC |=  (1<<LCD_RES);

#define CLR_WR PORTC &= ~(1 << LCD_WR);
#define SET_WR PORTC |=  (1 << LCD_WR);
#define CLR_RD PORTC &= ~(1 << LCD_RD);
#define SET_RD PORTC |=  (1 << LCD_RD);

/* color definition */                                  

#define	BLACK	  0x0000
#define	BLUE	  0x001F
#define	RED 	  0xF800
#define	GREEN     0x07E0
#define CYAN	  0x07FF
#define MAGENTA   0xF81F
#define YELLOW    0xFFE0
#define WHITE	  0xFFFF
#define OLIVE     0x8400
#define CHOCOLATE 0xD343

// define area color & text color

#define area_color 0xFFFF
#define text_color 0xF800

// 1st Row 
// Area 1
#define ves  0x0F00     // Vertical RAM address position End and Start (1st Row)

/* Description: delay */                                  

void delay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	for(t1=11;t1>0;t1--)
	{asm("nop");}
}
/* Description: write command    */                                  

void write_command(unsigned int command)
{
	CLR_CS;
	CLR_CD;
	SET_RD;
	SET_WR;

	PORTA=command;
	PORTE=command>>8;

	CLR_WR;
	SET_WR;
	SET_CS;
}

/* Description: write data */                                  

void write_data(unsigned int data)
{
	CLR_CS;
	SET_CD;
	SET_RD;
	SET_WR;

	PORTA=data;
	PORTE=data>>8;
	
	CLR_WR;
	SET_WR;
	SET_CS;

}
// Including character Pattern File
#include "fnt16x16.h"

/* Description: initialisation  */                                  

void initialization()
{
	SET_RD;
	SET_WR;
 	SET_CS;
	SET_CD;

	PORTA=0x00;
	PORTE=0x00;

	CLR_RESET;
	delay(200);
	SET_RESET;
	delay(500);

    write_command(0x0028);    // VCOM OTP
    write_data(0x0006);       // Page 55-56 of SSD2119 datasheet

	write_command(0x0000);    // start Oscillator
	write_data(0x0001);       // Page 36 of SSD2119 datasheet

	write_command(0x0010);    // Sleep mode
	write_data(0x0000);       // Page 49 of SSD2119 datasheet

	write_command(0x0001);    // Driver Output Control
	//write_data(0x32EF);       // Page 36-39 of SSD2119 datasheet (CFAF320240.c)
	write_data(0x72EF);       // Page 36-39 of SSD2119 datasheet

	write_command(0x0002);    // LCD Driving Waveform Control
	write_data(0x0600);       // Page 40-42 of SSD2119 datasheet

	write_command(0x0003);    // Power Control 1
	write_data(0x6A38);       // Page 43-44 of SSD2119 datasheet

	write_command(0x0011);    // Entry Mode
	write_data(0x6870);       // Page 50-52 of SSD2119 datasheet -- ID and AM -- write_data(0x6870);

	write_command(0X000F);    // Gate Scan Position  	--- write_command(0X000F);
	write_data(0x0000);       // Page 49 of SSD2119 datasheet --- write_data(0x0000); position  lhorizontale

	write_command(0x000B);    // Frame Cycle Control
	write_data(0x5308);       // Page 45 of SSD2119 datasheet

	write_command(0x000C);    // Power Control 2
	write_data(0x0003);       // Page 47 of SSD2119 datasheet

	write_command(0x000D);    // Power Control 3
	write_data(0x000A);       // Page 48 of SSD2119 datasheet

	write_command(0x000E);    // Power Control 4
	write_data(0x2E00);       // Page 48 of SSD2119 datasheet

	write_command(0x001E);    // Power Control 5
	write_data(0x00BE);       // Page 53 of SSD2119 datasheet

	write_command(0x0025);    // Frame Frequency Control
	write_data(0x8000);       // Page 53 of SSD2119 datasheet

	write_command(0x0026);    // Analog setting
	write_data(0x7800);       // Page 54 of SSD2119 datasheet

	write_command(0x004E);    // Ram Address Set
	write_data(0x0000);       // Page 58 of SSD2119 datasheet

	write_command(0x004F);    // Ram Address Set
	write_data(0x0000);       // Page 58 of SSD2119 datasheet

	write_command(0x0012);    // Sleep mode
	write_data(0x08D9);       // Page 49 of SSD2119 datasheet

	// Gamma Control (R30h to R3Bh) -- Page 56 of SSD2119 datasheet
	write_command(0x0030);
	write_data(0x0000);

	write_command(0x0031);
	write_data(0x0104);

	write_command(0x0032);
	write_data(0x0100);

	write_command(0x0033);
	write_data(0x0305);

	write_command(0x0034);
	write_data(0x0505);

	write_command(0x0035);
	write_data(0x0305);

	write_command(0x0036);
	write_data(0x0707);

	write_command(0x0037);
	write_data(0x0300);

	write_command(0x003A);
	write_data(0x1200);

	write_command(0x003B);
	write_data(0x0800);		 

	write_command(0x0007);    // Display Control  SPT = 1 to set 2nd screen
	write_data(0x0133);       // Page 45 of SSD2119 datasheet

	delay(150);

	write_command(0x0022);    // RAM data write/read
}

/* Description: display home */                                  

void Display_Home()
{
	write_command(0x004E);    // RAM address set
	write_data(0x0000);       // Page 58 of SSD2119 datasheet
	write_command(0x004F);    // RAM address set
	write_data(0x0000);       // Page 58 of SSD2119 datasheet

	write_command(0x0044);    // Vertical RAM address position 
	write_data(0xEF00);       // Page 57 of SSD2119 datasheet  Colum --- write_data(0xEF00);
	write_command(0x0045);    // Horizontal RAM address position 
	write_data(0x0000);       // Page 57 of SSD2119 datasheet  --- start positions write_data(0x0000);
	write_command(0x0046);    // Horizontal RAM address position
	write_data(0x013F);       // Page 57 of SSD2119 datasheet  --- end positions write_data(0x013F)

	write_command(0x0022);    // RAM data write/read
}

//  Description: background color 
void display_background_color(unsigned int color)
{
	unsigned int i,j;
	Display_Home();
	for(i=0;i<320;i++)
  {
  	for(j=0;j<240;j++)
  	{
  		write_data(color);
  	}
  }
}

// Converting hexa to decimal (Verticle Start-End)

unsigned char htod_vse(int s,char h)
 {
  //shift to right for VE, else s=0 thus no shifting takes place, i.e. shift by 0
	unsigned char p = s >> (8*h);	
 	return p; 
}

// declare explicite disp_char function
void disp_char16(char c);
void disp_str(char s[50]);

// Draw Area
void draw_area(unsigned int ve_vs, unsigned int hs, unsigned int he)
{
	int d=500; // For delay

	unsigned int i,j,dec[2];  //0 is LSB and 1 is MSB; i.e. 0 is VS and 1 is VE

	dec[0] = htod_vse(ve_vs,0); // Verticle Start Value
	dec[1] = htod_vse(ve_vs,1); // Verticle End Value
	
	write_command(0x004E);    // RAM address set
	write_data(hs);           // Page 58 of SSD2119 datasheet (X value)
	write_command(0x004F);    // RAM address set
	write_data(dec[0]);       // Page 58 of SSD2119 datasheet (Y value)
	write_command(0x0044);    // Vertical RAM address position 
	write_data(ve_vs);        // Page 57 of SSD2119 datasheet  Colum --- write_data(0xEF00);
	write_command(0x0045);    // Horizontal RAM address position 
	write_data(hs);           // Page 57 of SSD2119 datasheet  --- start positions write_data(0x0000);
	write_command(0x0046);    // Horizontal RAM address position
	write_data(he);           // Page 57 of SSD2119 datasheet  --- end positions write_data(0x013F)

	write_command(0x0022);    // RAM data write/read

	for(i=hs; i<=he; i++) // Row (Horizontal) loop
		{
			for(j=dec[0]; j<=dec[1]; j++) // Column (Verticle) loop
			{
				write_data(area_color);
			}
		}
	_delay_ms(d);

/*
	// Using Pointer [Memory management] but not worked

	unsigned int k;             //loop variable
	unsigned int *buffer;		//buffer

	buffer = malloc(512);
	if (buffer == 0)
	return;

	//write_command(0x0022);

	//k = 0; // byte count
	
	while (1)
	{
		for(i=hs; i<=he; i++) // Row (Horizontal) loop
		{
			for(j=dec[0]; j<=dec[1]; j++) // Column (Verticle) loop
			{
				write_data(area_color);
			}
		}

	}
	free(buffer);
*/
}
// Display Character of 16x16 font
void disp_char16(char c)
{
	switch(c)
	{	
// print A 16x16
		case 'A':
				patt(char_A16);			
				break;
 // print B 16x16
		case 'B':
				patt(char_B16);			
				break;
// print C 16x16
		case 'C':
				patt(char_C16);			
				break;
// print D 16x16
		case 'D':
				patt(char_D16);			
				break;
// print E 16x16
		case 'E':
				patt(char_E16);			
				break;
// print F 16x16
		case 'F':
				patt(char_F16);			
				break;
// print G 16x16
		case 'G':
				patt(char_G16);			
				break;
 // print H 16x16
		case 'H':
				patt(char_H16);			
				break;

// print I 16x16
		case 'I':
				patt(char_I16);			
				break;
// print J 16x16
		case 'J':
				patt(char_J16);			
				break;
// print K 16x16
		case 'K':
				patt(char_K16);			
				break;
// print L 16x16
		case 'L':
				patt(char_L16);			
				break;
// print M 16x16
		case 'M':
				patt(char_M16);			
				break;
// print N 16x16
		case 'N':
				patt(char_N16);			
				break;
// print O 16x16
		case 'O':
				patt(char_O16);			
				break;
// print P 16x16
		case 'P':
				patt(char_P16);			
				break;
// print Q 16x16
		case 'Q':
				patt(char_Q16);			
				break;
// print R 16x16
		case 'R':
				patt(char_R16);			
				break;
// print S 16x16
		case 'S':
				patt(char_S16);			
				break;
// print T 16x16
		case 'T':
				patt(char_T16);			
				break;
// print U 16x16
		case 'U':
				patt(char_U16);			
				break;
// print V 16x16
		case 'V':
				patt(char_V16);			
				break;
// print W 16x16
		case 'W':
				patt(char_W16);			
				break;
// print X 16x16
		case 'X':
				patt(char_X16);			
				break;

// print Y 16x16
		case 'Y':
				patt(char_Y16);			
				break;
// print Z 16x16
		case 'Z':
				patt(char_Z16);			
				break;

// print nothing
		default:
			write_data(text_color);
			break;
	}
}

void disp_str(char s[50])
{ 
    
	// Display String in 3rd Row
	unsigned int v=0x0F00,hs=0x0000,he=0x000F;
	int t,l=0;

	l=strlen(s);
	for(t=0 ; t<=l ; t++)
	{
		// Check Character 
		if(s[t] == 'A')
		{
				draw_area(v,hs,he);
                disp_char16('A');
		}
		else if(s[t] == 'B')
		{
				draw_area(v,hs,he);
                disp_char16('B');
		}
		else if (s[t] == 'C')
		{
			draw_area(v,hs,he);
             disp_char16('C');
		}
		else if (s[t] == 'D')
		{
			draw_area(v,hs,he);
             disp_char16('D');
		}
		else if (s[t] == 'E')
		{
			draw_area(v,hs,he);
             disp_char16('E');
		}
		else if (s[t] == 'F')
		{
			draw_area(v,hs,he);
             disp_char16('F');
		}
		else if (s[t] == 'G')
		{
			draw_area(v,hs,he);
             disp_char16('G');
		}
		else if (s[t] == 'H')
		{
			draw_area(v,hs,he);
             disp_char16('H');
		}
		else if (s[t] == 'I')
		{
			draw_area(v,hs,he);
             disp_char16('I');
		}
		else if (s[t] == 'J')
		{
			draw_area(v,hs,he);
             disp_char16('J');
		}
		else if (s[t] == 'K')
		{
			draw_area(v,hs,he);
             disp_char16('K');
		}
		else if (s[t] == 'L')
		{
			draw_area(v,hs,he);
             disp_char16('L');
		}
		else if (s[t] == 'M')
		{
			draw_area(v,hs,he);
             disp_char16('M');
		}
		else if (s[t] == 'N')
		{
			draw_area(v,hs,he);
             disp_char16('N');
		}
		else if (s[t] == 'O')
		{
			draw_area(v,hs,he);
             disp_char16('O');
		}
		else if (s[t] == 'P')
		{
			draw_area(v,hs,he);
             disp_char16('P');
		}
		else if (s[t] == 'Q')
		{
			draw_area(v,hs,he);
             disp_char16('Q');
		}
		else if (s[t] == 'R')
		{
			draw_area(v,hs,he);
             disp_char16('R');
		}
		else if (s[t] == 'S')
		{
			draw_area(v,hs,he);
             disp_char16('S');
		}
		else if (s[t] == 'T')
		{
			draw_area(v,hs,he);
             disp_char16('T');
		}
		else if (s[t] == 'U')
		{
			draw_area(v,hs,he);
             disp_char16('U');
		}
		else if (s[t] == 'V')
		{
			draw_area(v,hs,he);
             disp_char16('V');
		}
		else if (s[t] == 'W')
		{
			draw_area(v,hs,he);
             disp_char16('W');
		}
		else if (s[t] == 'X')
		{
			draw_area(v,hs,he);
             disp_char16('X');
		}
		else if (s[t] == 'Y')
		{
			draw_area(v,hs,he);
             disp_char16('Y');
		}
		else if (s[t] == 'Z')
		{
			draw_area(v,hs,he);
             disp_char16('Z');
		}
		// For digit
		else if (s[t] == '1')
		{
			draw_area(v,hs,he);
             disp_char16('1');
		}
		else
		{
			write_data(YELLOW);
		}
		// Check Horizontal End
		if(he < 303)
		{
			hs=he;
			he = he + 15;
			
		}
		else
		{
			/* I want character size 16x16 
			   For long string i want address of second row if first row is full then the line start from next row..
			  e.g if my VEVS value is 0x0F00 then next adress will be VS 0+15 = 0F & VE 0F+15 = 1E 
			  so next adress will be 0x1E0F in register 0x0044..
			  Hope u understand my problem
			*/
			v=0xEADB;
			//v=0x2D1E;
			//v=0x1E0F;
			hs=0x0000;
			he=0x000F;
		}
	}
}

/* main  */                                  

int main()
{
    PORTD = 0xF8;	    // pull-up on switches, all others of
	DDRD  = 0x0F;   	// VPP and led pins output

	DDRA = 0xFF;    	// low data port, all outputs
	DDRE = 0xFF;    	// low data port, all outputs
	
	PORTC = 0xFE;	    // all pins high
	DDRC  = 0xFE;	    // all outputs

	// Initialize the display
	initialization();
	
	//Display background color
   display_background_color(BLACK);

	/* I first draw_area of 16x16 with other color & call character pattern & draw it with other color..
	   I write code for all character pattern but it support only 8 character pattern after that it gives memory error..
	   so i think memory will not released 
	   Hope u understand the problem...
	*/
	draw_area(0xC2B3,0x0103,0x0112);
	disp_char16('D');
	
	disp_str("ABCD DCBA AAAA BBBB CCCC DDDD DCBA ABCD");
	disp_str("ABCDEFGH HGFEDCBA");

	while(1)
	{
	
	}
	return 0;
}
/*************************************************/

